from __future__ import annotations

from typing import Any

__version__: str
__author__: str

class Skprocess: ...
class Pool: ...
class Share: ...
class Pipe: ...
class ProcessTimers: ...
def autoreconnect(*args: Any, **kwargs: Any) -> Any: ...

class ProcessError(Exception): ...
class PreRunError(ProcessError): ...
class RunError(ProcessError): ...
class PostRunError(ProcessError): ...
class OnFinishError(ProcessError): ...
class ResultError(ProcessError): ...
class ErrorHandlerError(ProcessError): ...
class ProcessTimeoutError(ProcessError): ...
class ResultTimeoutError(ProcessError): ...

__all__: list[str]
