from __future__ import annotations

from typing import Any, Optional

def time() -> float: ...
def sleep(seconds: float) -> float: ...
def elapsed(time1: float, time2: Optional[float] = ...) -> float: ...

class Sktimer: ...

class TimeThis:
    def __init__(self, timer: Sktimer | None = ...) -> None: ...
    def __enter__(self) -> Sktimer: ...
    def __exit__(self, exc_type: Any, exc: Any, tb: Any) -> None: ...

def timethis(*args: Any, **kwargs: Any) -> Any: ...
def clear_global_timers() -> None: ...

__all__: list[str]
